ifndef cameraFree_wdl;
define cameraFree_wdl;

// ver: 2
// date: 040505



// title: Free Camera
// image: cameraFree01.pcx
// help: The Free Camera is an external camera that is under user control.
// help: Note: You must attach the Free Camera action to an entity in your level.
// help: To disable camera features (such as rolling the camera) simply
// help:assign 'nullvalues' to the key comands you don't want your
// help:players to have access to.


// uses: display_active_camera_vec
// needs: display00.wdl, miscInput01.wdl, cameraTarget.wdl
// needs: cameraSelect.wdl

// prefix: cameraFree01_
// idcode: 006



// General Camera Entries







// Free Camera Entries

// entry: ID
// help: Identification number for this camera (used in cameraSelect)
// help: NOTE: Action not defined if two camera share a single value
// id: 3
var	cameraFree01_id = 10;

// enable: Hide Camera Entity?
// id: 1
define	cameraFree01_hide_b;

// enable: preform collision test?
// id: 2
//define	cameraFree01_collide_b;

// enable: Make free camera selectable by user?
// id: 4
define	cameraFree01_user_selectable_b;

// entry: Is this an "avatar" camera?
// entry_help: Avatar cameras are cameras who's source/pan are those of the player's eyes
// id: 29
//define	cameraFree01_avatar_q;


// ------------------------------------------------------------
// section: Camera Forces

// entry: Move Force
// help: Force used to move camera (higher value = faster)
// id: 5
var	cameraFree01_force = 40;
// entry: Rotation Force
// help: Force used to rotate camera (higher value = faster)
// id: 6
var	cameraFree01_ang_force = 20;

// ------------------------------------------------------------
// section: Input Keys

// entry: Move forward key
// id: 7
string cameraFree01_move_for_key_c = "I";
var	cameraFree01_move_for_scancode = 0;
// entry: Move backwards key
// id: 8
string cameraFree01_move_back_key_c = "K";
var	cameraFree01_move_back_scancode = 0;
// entry: Move left key
// id: 9
string cameraFree01_move_left_key_c = "J";
var	cameraFree01_move_left_scancode = 0;
// entry: Move right key
// id: 10
string cameraFree01_move_right_key_c = "L";
var	cameraFree01_move_right_scancode = 0;
// entry: Move up key
// id: 11
string cameraFree01_move_up_key_c = "U";
var	cameraFree01_move_up_scancode = 0;
// entry: Move down key
// id: 12
string cameraFree01_move_down_key_c = "O";
var	cameraFree01_move_down_scancode = 0;

// entry: Pan left (counter-clockwise) key
// id: 13
string cameraFree01_pan_left_key_c = "";
var	cameraFree01_pan_left_scancode = 0;
// entry: Pan right (clockwise) key
// id: 14
string cameraFree01_pan_right_key_c = "";
var	cameraFree01_pan_right_scancode = 0;
// entry: Tilt forward key
// id: 15
string cameraFree01_tilt_for_key_c = "";
var	cameraFree01_tilt_for_scancode = 0;
// entry: Tilt back key
// id: 16
string cameraFree01_tilt_back_key_c = "";
var	cameraFree01_tilt_back_scancode = 0;
// entry: Roll clockwise key
// id: 17
string cameraFree01_roll_cw_key_c = "";
var	cameraFree01_roll_cw_scancode = 0;
// entry: Roll counter-clockwise key
// id: 18
string cameraFree01_roll_ccw_key_c = "";
var	cameraFree01_roll_ccw_scancode = 0;

// -------------------------------------------------------
// section: Mouse Control

// entry: Use mouse to control pan/tilt of camera?
// id: 19
define	cameraFree01_mouse_look_b;

// entry: Mouse Sensitivity
// help: Adjust mouse sensitivity (higher value = faster movement)
// id: 20
var	cameraFree01_mouse_force = 0.5;

// -------------------------------------------------------
// section: Joystick Control

// entry: Use Joystick to control camera movement and pan? (1 = yes, 0 = no)
// id: 21
define	cameraFree01_joy_cntl_b;

// entry: Deadzone
// help: create a 'deadzone' for the joystick (0-255).
// help: Input less then this value will be removed.
// cntl: spin 0 255 1
// id: 22
var	cameraFree01_joy_deadzone = 10;

// entry: Up Button
// help: Joystick button that moves camera up (0 = off, 1-10 = joy1 button)
// cntl: spin 0 10 1
// id: 23
var	cameraFree01_joy_up = 1;
// entry: Down Button
// help: Joystick button that moves camera down (0 = off, 1-10 = joy1 button)
// cntl: spin 0 10 1
// id: 24
var	cameraFree01_joy_down = 2;
// entry: Roll Clockwise
// help: Joystick button that rolls camera clockwise (0 = off, 1-10 = joy1 button)
// cntl: spin 0 10 1
// id: 25
var	cameraFree01_joy_roll_cw = 3;
// entry: Roll counter-clockwise
// help: Joystick button that rolls camera counter-clockwise (0 = off, 1-10 = joy1 button)
// cntl: spin 0 10 1
// id: 26
var	cameraFree01_joy_roll_ccw = 4;
// entry: Tilt Forward
// help: Joystick button that tilts camera forward (0 = off, 1-10 = joy1 button)
// cntl: spin 0 10 1
// id: 27
var	cameraFree01_joy_tilt_forward = 5;
// entry: Tilt Back
// help: Joystick button that tilts camera back (0 = off, 1-10 = joy1 button)
// cntl: spin 0 10 1
// id: 28
var	cameraFree01_joy_tilt_back = 6;


// ---------------------------------------------------------------------
// Local variables
var	cameraFree01_bit_check; 	// used in bit-compairs to check is this camera is active


starter	CameraFree01_Init
{
	diag("\nWDL-Loaded:cameraFree01.wdl");
}

entity* kamerafree;


// desc: Attach this function to the entity you wish to use as a free camera.
// desc: Additional free camera settings can be set in the "Free Camera" properties panel.
action	CameraFree01
{
	kamerafree=me;
	
	var	updated_b;	// has the camera been update yet this frame?
	var	view_number;	// the current view being concidered

 	// hide the camera model (if requested)
	ifdef cameraFree01_hide_b;
	//if(1 == cameraFree01_hide_b)
	//{
		MY.invisible = ON;
	//}
	//else
	ifelse;
	//{
		MY.invisible = OFF;
	//}
	endif;

	// make unpassable if collide is on
	ifdef cameraFree01_collide_b;
	//if(1 == cameraFree01_collide_b)
	//{
		MY.passable = OFF;
	//}
	ifelse;
	//else
	//{
		MY.passable = ON;
	//}
	endif;


	///////////////////////////
	// Set up joystick buttons
	///////////////////////////
	// set up joystick 'up' button
	if((cameraFree01_joy_up > 0) && (cameraFree01_joy_up <=10))
	{
		// scancode for joystick 1 buttons start at 256
		cameraFree01_joy_up += 255;
	}
	else
	{
		// negitive values not used
		cameraFree01_joy_up = -255;
	}
	// set up joystick 'down' button
	if((cameraFree01_joy_down > 0) && (cameraFree01_joy_down <=10))
	{
		// scancode for joystick 1 buttons start at 256
		cameraFree01_joy_down += 255;
	}
	else
	{
		// negitive values not used
		cameraFree01_joy_down = -255;
	}

	// set up joystick 'roll cw' button
	if((cameraFree01_joy_roll_cw > 0) && (cameraFree01_joy_roll_cw <=10))
	{
		// scancode for joystick 1 buttons start at 256
		cameraFree01_joy_roll_cw += 255;
	}
	else
	{
		// negitive values not used
		cameraFree01_joy_roll_cw = -255;
	}
	// set up joystick 'roll ccw' button
	if((cameraFree01_joy_roll_ccw > 0) && (cameraFree01_joy_roll_ccw <=10))
	{
		// scancode for joystick 1 buttons start at 256
		cameraFree01_joy_roll_ccw += 255;
	}
	else
	{
		// negitive values not used
		cameraFree01_joy_roll_ccw = -255;
	}

	// set up joystick 'tilt forward' button
	if((cameraFree01_joy_tilt_forward > 0) && (cameraFree01_joy_tilt_forward <=10))
	{
		// scancode for joystick 1 buttons start at 256
		cameraFree01_joy_tilt_forward += 255;
	}
	else
	{
		// negitive values not used
		cameraFree01_joy_tilt_forward = -255;
	}
	// set up joystick 'tilt back' button
	if((cameraFree01_joy_tilt_back > 0) && (cameraFree01_joy_tilt_back <=10))
	{
		// scancode for joystick 1 buttons start at 256
		cameraFree01_joy_tilt_back += 255;
	}
	else
	{
		// negitive values not used
		cameraFree01_joy_tilt_back = -255;
	}
	//////////////////////////////////////////////////////////////////////
	/////////////////////////////
	// Set up keyboard scancodes
	/////////////////////////////
	// use MiscInput01_Key_For_Str() to convert strings into scancodes
	// forward key
	str_cpy(miscInput01_temp_str,cameraFree01_move_for_key_c);
	cameraFree01_move_for_scancode = MiscInput01_Key_For_Str();
	// back key
	str_cpy(miscInput01_temp_str,cameraFree01_move_back_key_c);
	cameraFree01_move_back_scancode = MiscInput01_Key_For_Str();
	// right key
	str_cpy(miscInput01_temp_str,cameraFree01_move_right_key_c);
	cameraFree01_move_right_scancode = MiscInput01_Key_For_Str();
	// left key
	str_cpy(miscInput01_temp_str,cameraFree01_move_left_key_c);
	cameraFree01_move_left_scancode = MiscInput01_Key_For_Str();
	// up key
	str_cpy(miscInput01_temp_str,cameraFree01_move_up_key_c);
	cameraFree01_move_up_scancode = MiscInput01_Key_For_Str();
	// down key
	str_cpy(miscInput01_temp_str,cameraFree01_move_down_key_c);
	cameraFree01_move_down_scancode = MiscInput01_Key_For_Str();

	// pan left key
	str_cpy(miscInput01_temp_str,cameraFree01_pan_left_key_c);
	cameraFree01_pan_left_scancode = MiscInput01_Key_For_Str();
	// pan right key
	str_cpy(miscInput01_temp_str,cameraFree01_pan_right_key_c);
	cameraFree01_pan_right_scancode = MiscInput01_Key_For_Str();
	// Tilt forward key
	str_cpy(miscInput01_temp_str,cameraFree01_tilt_for_key_c);
	cameraFree01_tilt_for_scancode = MiscInput01_Key_For_Str();
	// Tilt back key
	str_cpy(miscInput01_temp_str,cameraFree01_tilt_back_key_c);
	cameraFree01_tilt_back_scancode = MiscInput01_Key_For_Str();
	// Roll clockwise key
	str_cpy(miscInput01_temp_str,cameraFree01_roll_cw_key_c);
	cameraFree01_roll_cw_scancode = MiscInput01_Key_For_Str();
	// Roll counter-clockwise key
	str_cpy(miscInput01_temp_str,cameraFree01_roll_ccw_key_c);
	cameraFree01_roll_ccw_scancode = MiscInput01_Key_For_Str();

	//////////////////////////////////////////////////////////////////////

	// init bit-field value used to check to see if this camera is active
	cameraFree01_bit_check = 1<<cameraFree01_id;

	ifdef cameraFree01_user_selectable_b;
	//if(1 == cameraFree01_user_selectable_b)
	//{
		// increase the camera select "max_camera" value if this value fall outside the current range
		if(cameraFree01_bit_check > cameraSelect_max_camera)
		{ cameraSelect_max_camera = cameraFree01_bit_check; }

		// add this camera type to the collection of valid cameras in camera select
		cameraSelect_valid_cameras |= cameraFree01_bit_check;
	//}
	endif;


	while(1)
	{
		updated_b = 0;	// camera hasn't been updated yet this frame
		view_number = 0;	// start with main view

		// for all active views
		while(view_number < display_active_views)
		{
			// check to see if this camera updates the current view
			if(display_active_camera_vec[view_number] & cameraFree01_bit_check)
			{
				// if the camera hasn't updated itself this frame...
				if(0 == updated_b)
				{
					// update camera position
					CameraFree01_Update();

					// camera has been updated this frame
					updated_b = 1;
				}

				// update current view
				CameraFree01_Update_View(view_number);

			}

			view_number +=1;
		}

		wait(1);	// every frame
	}


}


// desc: update the free camera
// uses: ME, temp, cameraFree01_force, cameraFree01_ang_force
// uses: cameraFree01_pan_left_key_c, cameraFree01_pan_right_key_c,
// uses: cameraFree01_forward_key_c, cameraFree01_backwards_key_c,
// uses:
function CameraFree01_Update()
{
	// make sure the player is visible to this camera
	if(cameraTarget_ent != NULL) { cameraTarget_ent.invisible = off; }

	// reset delta values
	// dx       // dy       // dz
	temp.x = 0; temp.y = 0; temp.z = 0;
	//pan          //tilt         // roll
	my.skill1 = 0; my.skill2 = 0; my.skill3 = 0;


	// check for keyboard input
	if(Key_Pressed(cameraFree01_pan_left_scancode))
	{
  		MY.skill1 += cameraFree01_ang_force*TIME;//pan
	}
	if(Key_Pressed(cameraFree01_pan_right_scancode))
	{
  		MY.skill1 -= cameraFree01_ang_force*TIME;//pan
	}
	if(Key_Pressed(cameraFree01_tilt_back_scancode))
	{
  		MY.skill2 += cameraFree01_ang_force*TIME;//tilt
	}
	if(Key_Pressed(cameraFree01_tilt_for_scancode))
	{
  		MY.skill2 -= cameraFree01_ang_force*TIME;//tilt
	}
	if(Key_Pressed(cameraFree01_roll_cw_scancode))
	{
  		MY.skill3 += cameraFree01_ang_force*TIME; //roll
	}
	if(Key_Pressed(cameraFree01_roll_ccw_scancode))
	{
  		MY.skill3 -= cameraFree01_ang_force*TIME; //roll
	}

	if(Key_Pressed(cameraFree01_move_for_scancode))
	{
  		temp.x += cameraFree01_force*TIME;
	}
	if(Key_Pressed(cameraFree01_move_back_scancode))
	{
  		temp.x -= cameraFree01_force*TIME;
	}
	if(Key_Pressed(cameraFree01_move_left_scancode))
	{
  		temp.y += cameraFree01_force*TIME;
	}
	if(Key_Pressed(cameraFree01_move_right_scancode))
	{
  		temp.y -= cameraFree01_force*TIME;
	}
	if(Key_Pressed(cameraFree01_move_up_scancode))
	{
  		temp.z += cameraFree01_force*TIME;
	}
	if(Key_Pressed(cameraFree01_move_down_scancode))
	{
  		temp.z -= cameraFree01_force*TIME;
	}

	// use mouse look?
	ifdef cameraFree01_mouse_look_b;
	//if(1 == cameraFree01_mouse_look_b)
	//{
	if (bEnableMouseInFreeCamera) //Intense Ai Mod
	{
		// add mouse forces
		MY.skill1 -= mickey.x*cameraFree01_mouse_force*TIME; // pan
		MY.skill2 -= mickey.y*cameraFree01_mouse_force*TIME; // tilt
	}
	//}
	endif;

	// use joystick?
	ifdef cameraFree01_joy_cntl_b;
	//if(1 == cameraFree01_joy_cntl_b)
	//{
		// forward/backwards movement
		if(abs(joy_raw.y) > cameraFree01_joy_deadzone)
		{
			temp.x -= (cameraFree01_force/255)*joy_raw.y*TIME;
		}

		// pan
		if(abs(joy_raw.x) > cameraFree01_joy_deadzone)
		{
			my.skill1 -= (cameraFree01_ang_force/255)*joy_raw.x*TIME; //pan
		}

		// move up?
		if(Key_Pressed(cameraFree01_joy_up))
		{
			temp.z += cameraFree01_force * TIME;
		}
		// move down?
		if(Key_Pressed(cameraFree01_joy_down))
		{
			temp.z -= cameraFree01_force * TIME;
		}

		// roll cw?
		if(Key_Pressed(cameraFree01_joy_roll_cw))
		{
			my.skill3 += cameraFree01_ang_force * TIME; //roll
		}
		// roll ccw?
		if(Key_Pressed(cameraFree01_joy_roll_ccw))
		{
			my.skill3 -= cameraFree01_ang_force * TIME; //roll
		}

		// tilt forward?
		if(Key_Pressed(cameraFree01_joy_tilt_forward))
		{
			my.skill2 -= cameraFree01_ang_force * TIME; //tilt
		}
		// tilt back?
		if(Key_Pressed(cameraFree01_joy_tilt_back))
		{
			my.skill2 += cameraFree01_ang_force * TIME;  //tilt
		}
	//}
	endif; // cameraFree01_joy_cntl_b

	// note: this is where you would normalize forces if you wanted to

	// add pan
 	vec_add(my.pan,my.skill1);

	// Move camera using forces
	move_mode = ignore_passable + ignore_passents + glide;
	Ent_Move(temp,nullvector);

}


// desc: update the view whos ID is passed in to the position/orientation
//      of the Free Camera
function	CameraFree01_Update_View(view_number)
{
	if(view_number == 0)
	{
		// update main view (camera)
		vec_set(camera.x,MY.x);
		vec_set(camera.pan,MY.pan);

		// is this an avatar view?
		ifdef cameraFree01_avatar_q;
			cameraTarget_avatar_view_q = 1;
		ifelse;
			cameraTarget_avatar_view_q = 0;
		endif;
	}

	// +++ other views


}

endif;


