/*********************************************************************
**									Intense Pathfinding                      **
**                         -------------------                      **
**                                             By Intense Illusions **
*********************************************************************/




/*********************************************
**********************************************
**              DATASTRUCTURES              **
**********************************************
*********************************************/

//prototypes
function Dijkstra(&aMatrix,startVertex,MatrixSize,MaximumNodes);
function GetShortestPathByIteration(&previous,&sPath,BeginNode,MaximumNodes);
function clearArray(&array,arraySize,cleanNum);


//Dijkstra (Preferably use these arrays when doing Dijkstra)
var Dijkstra_previous[MaxWayPoints];  //MaxNodes(which is also uses Dijsktra is way less than MaxWayPoints)
var Dijkstra_vertices[MaxWayPoints];  //same here
var Dijkstra_costs[MaxWayPoints];     //and here

define Dijkstra_Q,1;
define Dijkstra_S,2;





/*********************************************
**********************************************
**             DIJKSTRA FUNCTIONS           **
**********************************************
*********************************************/


//Dijkstra's usability is not limited to Pathfinding. 
//It may be used by you own applications when the shortest path between connected abstract nodes(don't need to be physical models at all) is required.




//Dijkstra Supporting function. Gets Path Found from Dijkstra, by looking at the previous array. sPath array is expected to be 2d
function GetShortestPathByIteration2dArray(&previous,&sPath,BeginNode,MaximumNodes)
{
	var u;
	var i;
	var TotalDistance;
	
	u=BeginNode;
	i=0;
	while (u!=undefined)
	{
		sPath[my.is_vIndex*MaxWayPoints+i]=u;
		i+=1;
		
		u=previous[u];
	}

	TotalDistance=Dijkstra_costs[sPath[my.is_vIndex*MaxWayPoints+0]];
	
	//If total distance is infinite, it means Dijkstra didn't found a Path
	if (TotalDistance==infinity) {return(undefined);}

	sPath[my.is_vIndex*MaxWayPoints+i]=undefined;
	
	return(TotalDistance);
}


//Dijkstra Supporting function. Returns if there are Any Q left in the Dijkstra Vertices array
function Dijkstra_AnyQ_Left(MatrixSize)
{
	var i=0;
	
	while (i<=MatrixSize)
	{
		if (Dijkstra_vertices[i]==Dijkstra_Q)
			{return (is_true);}
		i+=1;
	}
	
	return (is_false);
}


//Dijkstra Supporting function. Extract Minimum number in the Dijkstra vertices array
function Dijkstra_extract_Minimum(MatrixSize)
{
	var i=0;
	var MinNode=undefined;
	
	while (i<=MatrixSize)
	{
		if (Dijkstra_vertices[i]==Dijkstra_Q)
		{
			if (MinNode==undefined)
			{
				MinNode=i;
			}
			else
			{
				if (Dijkstra_costs[i]<Dijkstra_costs[MinNode])
				{
					MinNode=i;
				}
			}
		}
		i+=1;
	}
	
	return (MinNode);
}



//The Dijkstra function. Calculates all optimal Paths between a starting Node and every other node.
//aMatrix: any 2d array, marking the distances between any two nodes(Adjacency matrix)
//startVertex: Starting Node
//Matrix Size: The size of the matrix and also the number of nodes
//MaximumNodes: Maximum Nodes Supported by Pathfinding System
//To better understand the workings of Dijkstra, study a Dijkstra algorithm found in the web(ex Wikipedia)
	//Trying to understand how Dijkstra works from c-script will prove much harder
function Dijkstra(&aMatrix,startVertex,MatrixSize,MaximumNodes)
{
	var i;
	var u;
	
	
	//SET ALL ARRAYS

	i=0;
	while (i<=MatrixSize)
	{
		//Denote all Vertices in Dijkstra_vertices as Dijkstra_Q
		Dijkstra_vertices[i]=Dijkstra_Q;
		
		//Clear previous Array
		Dijkstra_previous[i]=undefined;
		
		//Have all costs as infinite for start
		Dijkstra_costs[i]=infinity;
		
		i+=1;	
	}
	
	Dijkstra_costs[startVertex]=0;
	
	while (Dijkstra_AnyQ_Left(MatrixSize)==is_true)
	{
		u=Dijkstra_extract_Minimum(MatrixSize);
		Dijkstra_vertices[u]=Dijkstra_S;
		
		i=0;
		while (i<=MatrixSize)
		{
			if (aMatrix[u*MaximumNodes+i]!=0)
			{
				if (Dijkstra_vertices[i]==Dijkstra_Q)
				{
					if (Dijkstra_costs[i]>Dijkstra_costs[u]+aMatrix[u*MaximumNodes+i])
					{
						Dijkstra_costs[i]=Dijkstra_costs[u]+aMatrix[u*MaximumNodes+i];
						Dijkstra_previous[i]=u;
					}
				}
			}
			i+=1;
		}
	}
}





/*********************************************
**********************************************
**            MISC DATASTRUCTURES           **
**********************************************
*********************************************/


//It is recommended that you study these functions and use them in your own projects.



//Get the Shortest Path by looking at their linked nodes. More Generic to work for any array. sPath array is expected to be 1d
function GetShortestPathByIteration(&previous,&sPath,BeginNode,MaximumNodes)
{
	var u;
	var i;
	
	//Clean Up array
	clearArray(sPath,MaximumNodes,undefined);
	
	u=BeginNode;
	i=0;
	while (u!=undefined)
	{
		sPath[i]=u;
		i+=1;
		
		u=previous[u];
	}
	
}




//Queue Datastructure


//Push a Value into the Queue
function Queue_Push(&Queue,Node)
{
	var i=0;
	
	while (Queue[i]!=0) {i+=1;}
	Queue[i]=Node;
}

//Pull a Value from the Queue
function Queue_Pull(&Queue)
{
	var pulled;
	var i=1;
	
	pulled=Queue[0];
	Queue[0]=0;
	
	while (Queue[i]!=0) 
	{
		Queue[i-1]=Queue[i];
		Queue[i]=0;
		i+=1;
	}

	return (pulled);
}


//Returns TRUE if QUEUE is empty, and false othewise
function Queue_isEmpty(&Queue)
{
	if (Queue[0]==0) {return (IS_TRUE);}
		else {return (IS_FALSE);}
}









//Array Traverse



//Extract minimum value from 'array' and replace it with 'Replacement'
function ExtractMinimumFrom(&array,ArraySize,Replacement)
{
	var MinNumber=infinity;
	var MinNumberIndex=undefined;
	
	var i=0;
	while (i<=ArraySize)
	{
		if (array[i]<MinNumber)
		{
			MinNumber=array[i];
			MinNumberIndex=i;
		}
		
		i+=1;	
	}
	
	if (Replacement!=undefined)
	{
		array[MinNumberIndex]=Replacement;
	}
	
	return(MinNumberIndex);
}



//returns true if 'value' is found in 'array' and false otherwise
function LookUpValueFromArray(&array,value)
{
	var i=0;
	
	while (array[i]!=0)
	{
		if (array[i]==value) {return (i);}
		i+=1;
	}
	
	return(notFound);
}


//Clears 'array' by replacing all cells with 'clearNum'
function clearArray(&array,arraySize,cleanNum)
{
	var i=0;
	while (i<arraySize)
	{
		array[i]=cleanNum;
		i+=1;
	}
}
