/*********************************************************************
**									Intense Pathfinding                      **
**                         -------------------                      **
**                                             By Intense Illusions **
*********************************************************************/




/*********************************************
**********************************************
**                WED ACTIONS               **
**********************************************
*********************************************/



//Wed Action for the Waypoint
action is_ai_Waypoint
{
	//Register WayPoint!!!
	RegisterWayPoint();

	//Make the Visible if at Graph_Making mode
	ifdef MAKE_GRAPH;
		my.invisible=off;
	ifelse;
		my.invisible=on;
	endif;

	//Make it Passable
	my.passable=on;
	my.push=-1; //Higher than Dynamic Entities to ignore them when Dropig down from gravity, but higher than zero to consider model-only levels
}


//Wed Action for the Green(or Red) X
action is_ai_Debug_TP
{
	Debug_tp=my;
	
	//Make it Passable
	my.passable=on;
	my.push=-3;
	
	//Pass It's position as parameters
	TargetPosX=my.x;
	TargetPosY=my.y;
	TargetPosZ=my.z;
	
	//Make it Turn
	while (1)
	{
		my.pan+=3;
		wait(1);
	}
}




//----------------------------------------------------------------------
// action: is_ai_Npc
// Desc: is_ai_Npc
//
//------------------
// section: Movement
// help: Forward force used to calculate AI speed.
// cntl: spin 0 x 0.25
// skill3: ForceX 3.5
//
// help: Force used to calculate AI "side-step" speed.
// cntl: spin 0 x 0.25
// skill4: ForceY 1.75
//
// help: Force used to calculate the AI's rotation speed.
// cntl: spin 0 x 0.25
// skill6: Pan 10
//
//------------------
// section: Limits
//
// help: Used to adjust the amount of upward sliding the AI can do.
// help: 0-can slide up onto anything under half its height, 0.99 no upward sliding.
// cntl: spin 0 0.99 0.05
// skill9: MinMoveZ 0.55
//
// ----------------------------------------
// section: Intense Ai
// help: This Entity will be respopsible for making the Pathfinding Graph
// flag6: Graph_Maker
action is_ai_Npc
{
	ifdef	GID01_OLD_COLLIDE;	// use the old collision code
		my.fat = OFF; my.narrow = ON;
	ifelse;	// use A6 collison
		my.fat = ON; my.narrow = ON;
		c_setminmax(my);
	endif;
		my.polygon=off;
	
	//Bot Responsible of Creating the Graph
	if (!is_ai_GraphAssigned && my.is_fGraphMaker==on)
	{
		is_ai_GraphAssigned=IS_TRUE;
		wait(10); //wait for all waypoints to build first and all dynamic entities to destroy
		ifdef MAKE_GRAPH;
			//If at Graph_Making mode, since this bot is responsible for the graph, generate it
			Generate_Graph();
			//After it's done, halt here
			while (1) {wait(1);}
		ifelse;
			//If this Bot is responsible for the Graph but in Normal mode(Not graph_Making), load the Graph from the .gph file to an array
			Load_Graph();
		endif;
	}
	else//For all other Bots
	{	
		ifdef MAKE_GRAPH;
			//If at Graph Making mode, remove any bots not responsible for generating/loading the Graph
			ent_remove(me);
			return;
		endif;
	}

	//Cast Shadow
	ShadowMe();

	//Very Important! This will make Dynamic entities ignored by other dynamic entities(which will be incremented by one) when casting rays
	my.push=-2;

	//Get a Unique Id, which will be used for identifying this entity(used to simutaneously pathfind many bots at once)
	RegisterMe();


	//Loop While I'm active
	while (my!=null)
	{
		//MOVEMENT
		
		//This is a Very Important line. If you want to use your own Movement Function(recommended), do something similar to pass
		//The movement intentions(generated by the Pathfinding Code, to your Movement Function)
		//All that the Pathfinding Code does, goes down into altering the my.boostVectorX,Y,Z and my.PanForce
		is_ai_BipedPhy01_Update(my.boostVectorX, my.PanForce);	//Interface between Pathfinding and Physics Movement
		
		
		//After the Walking Forces are applied to physics, reset to null
		vec_set(my.boostVectorX,nullvector);
		my.PanForce=0;
		
		
		// ANIMATION
		
		//Very very simple animation code in order keep the code clean and focused on Pathfinding.
		//Not at all recommended to keep this for your project, it only supports walking and standing.
		MY._biped01_anim_dist+=time*10;
		if (my._biped01_anim_dist>100) {MY._biped01_anim_dist=0;}

		if (my.is_cMoveMode==is_cStopped)	{ent_animate(me,"stand",MY._biped01_anim_dist,ANM_CYCLE);}
			else	{ent_animate(me,"run",MY._biped01_anim_dist,ANM_CYCLE);}
			
			
		wait(1);
	}
}





/*********************************************
**********************************************
**              GRAPH MAKER FUNCTIONS       **
**********************************************
*********************************************/


//Force 800x600 Resolution in Graph_Making mode
//To make sure the Interface Panels fit the screen
ifdef MAKE_GRAPH;
	var   video_mode = 7;
endif;


//Bmaps
bmap bGraphInProgress = <is_ai_Graph_Progress.pcx>;
bmap bGraphFinished = <is_ai_Graph_Finished.pcx>;
bmap bGraphBar = <is_ai_Graph_Bar.pcx>;


//Fonts
font Graph_font = "Arial",1,16; // truetype font


//Texts
text tGraphTimeEstimated
{
  layer = 1001;
  pos_x = 340;
  pos_y = 397;
  size_y = 20;
  offset_y = 0;
  strings = 1;
  
  red = 255;
  green = 255;
  blue = 255; 

  font = Graph_font;
  string = sGraphTimeEstimated;
  alpha	= 90;
  flags transparent;
} 



text tGraphTimeRemaining
{
  layer = 1001;
  pos_x = 340;
  pos_y = 415;
  size_y = 20;
  offset_y = 0;
  strings = 1;
  
  red = 255;
  green = 255;
  blue = 255; 

  font = Graph_font;
  string = sGraphTimeRemaining;
  alpha	= 90;
  flags transparent;
} 





//Panels
panel pGraphInProgress
{
	bmap bGraphInProgress;
	layer=1000;
	pos_x=0;
	pos_y=0;
	alpha=90;
	flags d3d,refresh,transparent;
	
	digits 328,363,5,Graph_font,1,NumOfWayPoints;
	digits 328,381,5,Graph_font,1,NumOfWayEdges;
}


panel pGraphFinished
{
	bmap bGraphFinished;
	layer=1000;
	pos_x=0;
	pos_y=0;
	alpha=90;
	flags d3d,refresh,transparent;
	on_click=pGraph_ActionListener;
	
	digits 328,363,5,Graph_font,1,NumOfWayPoints;
	digits 328,381,5,Graph_font,1,NumOfWayEdges;
}




//Pointers
panel* pGraphBar;



//string
string pGraphBar_prototype= "window = 0,0,008,40,bGraphBar,0,0; pos_x=275; pos_y=517; alpha=90; flags d3d,refresh,transparent,visible;";
string pGraphBar_def[500];





//Functions
function pGraph_ActionListener()
{
	if (pointer.x>425 && pointer.x<555 && pointer.y>468 && pointer.y<501) 
	{
		str_cpy(sRunningMode,"Graph Viewer\nNavigate with I J K L\nExit with ESC");
		tGraphTimeEstimated.visible=off;
		tGraphTimeRemaining.visible=off;
		pGraphFinished.visible=off;
		mouse_toggle();
		bEnableMouseInFreeCamera=IS_TRUE;
	}
	if (pointer.x>585 && pointer.x<716 && pointer.y>468 && pointer.y<501) {exit;}

}




function GraphBar_Percent(percent)
{
	var maxSizeX=518;
	var maxsizeY=40;
	var SizeX;
	
	SizeX=int(is_ai_ZeroToXScale(percent,100,maxSizeX));
		
	
	if (pGraphBar!=null) {pGraphBar.visible=off;}
	
	if (SizeX!=0) {str_for_num(is_sTemp3,SizeX);}
		else {str_cpy(is_sTemp3,"1");}
	
	
	str_cpy(pGraphBar_def,pGraphBar_prototype);
	
	str_clip(pGraphBar_def,16);
	
	str_cpy(is_sTemp4,pGraphBar_def);
	
	str_cpy(pGraphBar_def,"window = 0,0,");
	str_cat(pGraphBar_def,is_sTemp3);
	str_cat(pGraphBar_def,is_sTemp4);
	

	pGraphBar = pan_create(pGraphBar_def,1001);
	
}


function Generate_Graph()
{	
	is_ai_bMakeGraphMode=IS_TRUE;
	bEnableMouseInFreeCamera=IS_FALSE;
	
	//Set Camera to Graph_maker's eyes
	you=kamerafree;
	vec_set(you.x,my.x);
	you.z+=my.max_z+30;
	vec_set(you.pan,my.pan);

	//Halt All music and sounds
	media_stop(0);

	//Make sure you Halt ALL other functions running.
	//...

	//Switch to Free Camera
	CameraSelect_Next();

	//Panels
	str_cpy(sRunningMode,"");
	pGraphInProgress.visible=on;
	GraphBar_Percent(0);
	tGraphTimeEstimated.visible=on;
	tGraphTimeRemaining.visible=on;
	str_cpy(sGraphTimeEstimated,"Estimating..");
	str_cpy(sGraphTimeRemaining,"Estimating..");
	wait(3);
	WayPoints_Gravity(waypoint_count);
	Create_PoVGraph(waypoint_count); //For Points of Visibility Graph Creation
	
	while (!bGraphDone) {wait(1);}
	ExportGraphToFile(waypoint_count);
	
	pGraphInProgress.visible=off;
	pGraphFinished.visible=on;
	pGraphBar.visible=off;
	str_cpy(sGraphTimeRemaining,"Finished!");

	mouse_toggle();
}


function Load_Graph()
{
	WayPoints_Gravity(waypoint_count);
	ImportGraphFromFile(waypoint_count); //Import Graph From GPH File
}




/*********************************************
**********************************************
**                CONSOLE                   **
**********************************************
*********************************************/


//Fonts
font FunnyConsoleFont = "arial",1,20; // truetype font 
    
    
//Texts
text FunnyConsole
{
  layer = 500;
  pos_x = 5;
  pos_y = 1;
  size_y = 60;
  offset_y = 0;
  strings = 1;
  
  red = 75;
  green = 82;
  blue = 125; 

  font = FunnyConsoleFont;
  string = sRunningMode;
  alpha	= 100;
  flags = shadow,outline,visible;
} 


//Functions
function ToggleDebugMode()
{
	if (!is_ai_bMakeGraphMode)
	{
		if (is_ai_bSuperDebugMode)
		{
			//Go To Normal Mode
			is_ai_bDebugMode=IS_FALSE;
			is_ai_bSuperDebugMode=IS_FALSE;
			ToggleWaypoints(IS_FALSE);
			str_cpy(sRunningMode,"Intense PathFinding - Normal\nPress 'Enter' for Debug Mode");
				//Debug-->Kill previous Red lines
				KillLines=1; wait(1); KillLines=0;
		}
		else
		{
			if (is_ai_bDebugMode)
			{
				//Go To Super Debug Mode
				is_ai_bDebugMode=IS_TRUE;
				is_ai_bSuperDebugMode=IS_TRUE;
				ToggleWaypoints(IS_TRUE);
				str_cpy(sRunningMode,"Intense PathFinding - Super Debug\nPress 'Enter' for Normal Mode");			
			}
			else
			{
				//Go To Debug Mode
				is_ai_bDebugMode=IS_TRUE;
				is_ai_bSuperDebugMode=IS_FALSE;
				ToggleWaypoints(IS_FALSE);
				str_cpy(sRunningMode,"Intense PathFinding - Debug\nPress 'Enter' for Super Debug Mode");			
			}
		}
	}	
	
}


//Events
on_enter=ToggleDebugMode;