/*********************************************************************
**									Intense Pathfinding                      **
**                         -------------------                      **
**                                             By Intense Illusions **
*********************************************************************/




/*********************************************
**********************************************
**           PATHFINDING FUNCTIONS          **
**********************************************
*********************************************/



//prototype functions
function AddGraphEdgeToMatrix(prevNode,nextNode,cost);


//Arrays
var PoVMatrix[999999];   //1000 WayPoints * 1000 WayPoints
var PoV_PathsDataBase[999999]; //is_cMaxEntities(1000) * MaxWayPoints(1000)

//Array Syntax
//PoV_PathsDataBase[my.is_vIndex*MaxWayPoints+WayPointId];





//Very Important function. The backbone of the Pathfinding routine
//Determines if Npc can walk from SourcePos To Target Pos
//Instead of doing an unreliable c_trace to determine LOS
//an iteration of c_traces are occured in order to simulate the Npc
//Actually walking along the path between the two points
//Gravity, Size of the Npc model, Npc's Speed and Npc's Ability to climb steps are all considered
var TestLink_SourcePos[3];
var TestLink_TargetPos[3];
function TestWalkLinkBetweenTwoPoints(&SourcePos,&TargetPos,vStepFront,eIgnored)
{
	var bReachedGoal=IS_FALSE;
	var vTotalDistance=0;
	var vDistanceCovered=0;
	var vAngle;
	var vecCurPoint[3];
	var vecNextPoint[3];
	
	var c_trace_mode;
	var c_trace_mode_no_box;
	
	var vStepUp;
	
	vStepUp=is_meter*0.5;
	
	var vBoundingBox;
	
	var MyOriginHeight;
	
	MyOriginHeight=my.min_z*(-1)+5;
	
	
	vBoundingBox=max(max(max(my.min_y,my.max_y),my.min_x),my.max_x);

	
	var OriginalPushValue;

	OriginalPushValue=my.push;

	//Fix trace_modes depending on eIgnored Mode (eIgnored may be either a constant or a pointer)
	
	//You will effeciently make use of this mode, if you first build a collision avoidance layer in addition to your long-tem pathfinding
	//The Longer Term Pathfinding should only consider STATIC level blocks or MDLs
	//The Obstacle Avoidance should only consider DYNAMIC models
	//With the absence of an obstacle avoidance algorithm, this mode now considers all models when pathfinding
	//To Have Long-Term Pathfinding ignore other dynamic entities, restore the comments in the if clause below)
	if (eIgnored==is_cIgnoreAllDynamicModels)
	{
		//my.push+=1;//This will increment this Dynamic Entity's push value one more than all other Dynamic models
		c_trace_mode=ignore_me+ignore_passable+ignore_passents+use_box;//+ignore_push;
		c_trace_mode_no_box=ignore_me+ignore_passable+ignore_passents;//+ignore_push;
		eIgnored=null;
	}
	else
	{
		//Don't ignore any models
		if (eIgnored==is_cIgnoreNoModels)
		{
			c_trace_mode=ignore_me+ignore_passable+use_box;
			c_trace_mode_no_box=ignore_me+ignore_passable;
			eIgnored=null;
		}
		else
		{
			//Ignore the You model
			c_trace_mode=ignore_me+ignore_you+ignore_passable+use_box;
			c_trace_mode_no_box=ignore_me+ignore_you+ignore_passable;
		}
	}
	
	//Copy SourcePos and TargetPos to Global arrays, in order to be able to be passed as parameters in other functions
	TestLink_SourcePos.x=SourcePos[0];
	TestLink_SourcePos.y=SourcePos[1];
	TestLink_SourcePos.z=SourcePos[2];
	
	TestLink_TargetPos.x=TargetPos[0];
	TestLink_TargetPos.y=TargetPos[1];
	TestLink_TargetPos.z=TargetPos[2];

	//Get Real Distance between the two Positions
	vTotalDistance=vec_dist(vector(TestLink_SourcePos.x,TestLink_SourcePos.y,0),vector(TestLink_TargetPos.x,TestLink_TargetPos.y,0));
	
	//Get the Turning Angle from Source Point to Target Point
	vAngle=is_ReturnAngleForTurningPosToPos(TestLink_SourcePos,TestLink_TargetPos);
	
	//Set Source Point as the current one(for further examinations further on)
	vec_set(vecCurPoint.x,TestLink_SourcePos.x);



	While (1) //This Loop will only exit in a return
	{
		//Mark Distance Covered so far in the 'simulated walk' between the two points
		vDistanceCovered+=vStepFront;
		
		//If the whole Distance is covered, mark that the Goal Point is reached
		if (vDistanceCovered>=vTotalDistance)
		{
			vDistanceCovered=vTotalDistance;
			bReachedGoal=IS_TRUE;
		}
		
		//Get a Point Inbetween, according to the simulated 'walk step forward'
		is_GetPosVecByRadious(TestLink_SourcePos.x,vAngle,vDistanceCovered,vecNextPoint.x);
		
		//Position Found is now the Next Point
		
		//Next Point's Altitude is set to the Current Point's one
		vecNextPoint.z=vecCurPoint.z;
		
		//Ignore You in the c_traces
		you=eIgnored;
		
			//Debug-->Visualise C_trace
			//vec_set(is_tempvec2.x,vecCurPoint.x);
			//vec_set(is_tempvec3.x,vecNextPoint.x);
			//c_trace_debug(is_tempvec2.x,is_tempvec3.x,c_trace_mode);
			
		//Trace between Current and Next Point
		c_trace(vecCurPoint.x,vecNextPoint.x,c_trace_mode);


		//If No Obstacles Found
		if (!trace_hit)
		{
			//Calculate Gravity for Simulated Walk
			
			//Ignore You
			you=eIgnored;
						
				//Debug-->Visualise C_trace
				//vec_set(is_tempvec2.x,vecNextPoint.x);
				//vec_set(is_tempvec3.x,vector(vecNextPoint.x,vecNextPoint.y,vecNextPoint.z-min(is_ai_MaximumFall,is_ai_DownwardsTrace)));
				//vecNextPoint.z-=max(0,c_trace_debug(is_tempvec2.x,is_tempvec3.x,c_trace_mode_no_box));
				
			//Do Downwards Gravity trace. Drop Next Point Down To the Ground (when simulating walking down the stairs for example)
			vecNextPoint.z-=max(0,c_trace(vecNextPoint.x,vector(vecNextPoint.x,vecNextPoint.y,vecNextPoint.z-min(is_ai_MaximumFall,is_ai_DownwardsTrace)),c_trace_mode_no_box));
			
			//Since this trace is no_box, the trace went all the way to the ground. Lift it to Npc's Height
			vecNextPoint.z+=MyOriginHeight;
			
			//Next Point is now the Current Point
			vec_set(vecCurPoint.x,vecNextPoint.x);
			
			if (!trace_hit) //If no floor found
			{
				//it must be a hole/opening so give up on attempting a Link(assume Bot can't jump)
				my.push=OriginalPushValue;
				return(undefined);  //Level Infinite Gap Found
			}
		}
		//If An obstacle is found between Current and Next Point, Simulate a StepUp between Current and Next Point
			//Indicating the Npc's ability to climb steps, slopes, etc
		else
		{
			//Step Up
			vecCurPoint.z+=vStepUp;
			vecNextPoint.z+=vStepUp;
			
			//Ignore You
			you=eIgnored;
			
				//Debug-->Visualise C_trace
				//vec_set(is_tempvec2.x,vecCurPoint.x);
				//vec_set(is_tempvec3.x,vecNextPoint.x);
				//c_trace_debug(is_tempvec2.x,is_tempvec3.x,c_trace_mode);
				
			//Do Another Trace
			c_trace(vecCurPoint.x,vecNextPoint.x,c_trace_mode);
			
			//If No Obstalces found in this second trace
			if (!trace_hit)
			{
				//Calculate Gravity for Simulated Walk
				
				//Ignore You
				you=eIgnored;
				
					//Debug-->Visualise C_trace
					//vec_set(is_tempvec2.x,vecNextPoint.x);
					//vec_set(is_tempvec3.x,vector(vecNextPoint.x,vecNextPoint.y,vecNextPoint.z-min(is_ai_MaximumFall,is_ai_DownwardsTrace)));
					//vecNextPoint.z-=max(0,c_trace_debug(is_tempvec2.x,is_tempvec3.x,c_trace_mode_no_box));
					
				//Do Downwards Gravity trace. Drop Next Point Down To the Ground (when simulating walking down the stairs for example)
				vecNextPoint.z-=max(0,c_trace(vecNextPoint.x,vector(vecNextPoint.x,vecNextPoint.y,vecNextPoint.z-min(is_ai_MaximumFall,is_ai_DownwardsTrace)),c_trace_mode_no_box));
				
				//Since this trace is no_box, the trace went all the way to the ground. Lift it to Npc's Height
				vecNextPoint.z+=MyOriginHeight;
				
				//Next Point is now the Current Point
				vec_set(vecCurPoint.x,vecNextPoint.x);
				
				if (!trace_hit) //If no floor found
				{
					//it must be a hole/opening so give up on attempting a Link(assume Bot can't jump)
					my.push=OriginalPushValue;
					return(undefined);  //Level Infinite Gap Found
				}
			}
			//If Another Obstacle is found in the second trace with the simulated Step Up(quite possible a tall wall or a steep slope which the NPC will be unable to climb)
			else
			{
				//In case the trace didn't failed because of a tall wall ahead, but because NPC's box is inside a wall
				//Do another trace to figure out which of the two cases occured
				if (vBoundingBox>=vTotalDistance-vDistanceCovered ||
				 	vec_dist(vector(TestLink_SourcePos.x,TestLink_SourcePos.y,0),vector(vecCurPoint.x,vecCurPoint.y,0))<=vBoundingBox)
				{
					//Drop the Current and Next points one step back(not trying to climb anything anymore
					vecCurPoint.z-=vStepUp;
					vecNextPoint.z-=vStepUp;
					
					//Ignore You
					you=eIgnored;
					
						//Debug-->Visualise C_trace
						//vec_set(is_tempvec2.x,vecCurPoint.x);
						//vec_set(is_tempvec3.x,vecNextPoint.x);
						//c_trace_debug(is_tempvec2.x,is_tempvec3.x,c_trace_mode_no_box);
						
					//Do the trace again WITHOUT using use_box(just a line)
					c_trace(vecCurPoint.x,vecNextPoint.x,c_trace_mode_no_box);
						
					//If No obstacle is found
					if (!trace_hit)
					{
						//Calculate Gravity for Simulated Walk
						
						//Ignore you
						you=eIgnored;
						
							//Debug-->Visualise C_trace
							//vec_set(is_tempvec2.x,vecNextPoint.x);
							//vec_set(is_tempvec3.x,vector(vecNextPoint.x,vecNextPoint.y,vecNextPoint.z-min(is_ai_MaximumFall,is_ai_DownwardsTrace)));
							//vecNextPoint.z-=max(0,c_trace_debug(is_tempvec2.x,is_tempvec3.x,c_trace_mode_no_box));
							
						//Do Downwards Gravity trace. Drop Next Point Down To the Ground (when simulating walking down the stairs for example)
						vecNextPoint.z-=max(0,c_trace(vecNextPoint.x,vector(vecNextPoint.x,vecNextPoint.y,vecNextPoint.z-min(is_ai_MaximumFall,is_ai_DownwardsTrace)),c_trace_mode_no_box));
						
						//Since this trace is no_box, the trace went all the way to the ground. Lift it to Npc's Height
						vecNextPoint.z+=MyOriginHeight;
						
						//Next Point is now the Current Point
						vec_set(vecCurPoint.x,vecNextPoint.x);
						
						
						if (!trace_hit) //If no floor found
						{
							//it must be a hole/opening so give up on attempting a Link(assume Bot can't jump)
							my.push=OriginalPushValue;
							return(undefined);  //Level Infinite Gap Found
						}
					}
					//Both First trace attempt, and second trace attempt(one step higher) failed, and third no_box trace failed as well(Beginning or End of a path)
					else
					{
						my.push=OriginalPushValue;
						return (undefined);  
					}
				}
				//Both First trace attempt, and second trace attempt(one step higher) failed, and it's not at beginning or end of path
				else
				{
					my.push=OriginalPushValue;
					return (undefined);
				}
			}
		}
		
		//If Goal Point is Reached
		if (bReachedGoal)
		{
			//Ignore You
			you=eIgnored;
			
				//Debug-->Visualise C_trace
				//vec_set(is_tempvec2.x,vecNextPoint.x);
				//vec_set(is_tempvec3.x,TestLink_TargetPos.x);
				//c_trace_debug(is_tempvec2.x,is_tempvec3.x,c_trace_mode_no_box);
				
			//Do a last linear trace(No use_box), to determine there's LOS between the point reached by simulating the NPC walk, and the actual walk Point
			c_trace(vecNextPoint.x,TestLink_TargetPos.x,c_trace_mode_no_box);
			
			//No obstacles found
			if (!trace_hit)
			{
				my.push=OriginalPushValue;
				return(vTotalDistance); //Path Found!
			}
			//No LOS exists which means the simulated walk didn't made it to the desired Goal Point(maybe the simulated walk 'fell down a bridge' and as it 'walked forward' it resulted in a whole different place
			else
			{
				my.push=OriginalPushValue;
				return(undefined);  //Final trace failed
			}
		}
		
	}
	
}


//Drop Down Waypoints to the Ground
function WayPoints_Gravity(Total_WayPoints)
{
	var MyOriginHeight;
	var i;
	
	MyOriginHeight=my.min_z*(-1);

	//For All Waypoints Registered
	i=0;
	while (i<=Total_WayPoints)
	{	
		//Lift/Lower Waypoint to model's Origin Point Altitude
		
		//Get WayPoint's Pointer
		is_tempent2=waypoint_to_ptr(i);
		vec_set(is_temp2,is_tempent2.x);
		
		//Find Position Below
		is_temp2.z-=is_ai_DownwardsTrace;
		
		//Do the Trace
		c_trace(is_tempent2.x,is_temp2.x,ignore_me+ignore_passable+ignore_passents+ignore_push);
		
		//Get Position from collision occured in c_trace
		vec_set(is_temp3.x,target.x);
		
		//Lift WayPoint up the Model's Height
		is_temp3.z+=MyOriginHeight;
		vec_set(is_tempent2.x,is_temp3.x);
		
		i+=1;	
	}
}




//Clamp SourcePos According to height, On the Ground, and return to ReturnedPos Vector
var ClampPosition_Source[3];
var ClampPosition_Target[3];
var ClampPosition_Return[3];
function ClampPosition(&SourcePos,Height,&ReturnedPos)
{
	//Copy Parameters to Global Vars
	vec_set(ClampPosition_Source,SourcePos);
	vec_set(ClampPosition_Target,SourcePos);
	
	//Find a Point Below
	ClampPosition_Target.z-=is_ai_DownwardsTrace;
	c_trace(ClampPosition_Source.x,ClampPosition_Target.x,ignore_me+ignore_passable);
	if (!trace_hit) //If No Floor found while clamping, it means point examined is probably not valid(out of level)
	{
		vec_set(ReturnedPos,SourcePos);//So Just return it as it is(no clamping)
		return;
	}
	
	//Raise Position From ground up to Height
	vec_set(ClampPosition_Return.x,target.x);
	ClampPosition_Return.z+=Height;
	
	//Return Clamped Position
	vec_set(ReturnedPos,ClampPosition_Return);
}





/*********************************************
**********************************************
**           MAKE GRAPH FUNCTIONS           **
**********************************************
*********************************************/



//Calculate Time Remaining
function EstimateTimeRemaining(EstimatedBuildInTicks)
{
	var EstimatedBuildInSeconds;
	var EstimatedBuildInMinutes;
	var EstimatedBuildInHours;
	
	EstimatedBuildInSeconds=int(EstimatedBuildInTicks/is_SecondInTicks);
	EstimatedBuildInMinutes=int(EstimatedBuildInSeconds/MinuteInSeconds);
	EstimatedBuildInHours=int(EstimatedBuildInMinutes/HourInMinutes);
	
	EstimatedBuildInSeconds-=EstimatedBuildInMinutes*MinuteInSeconds;
	EstimatedBuildInMinutes-=EstimatedBuildInHours*HourInMinutes;
	
	
	str_cpy(is_sTemp3,"");
	
	//Get Hours
	if (EstimatedBuildInHours>0)
	{
		str_for_num(is_sTemp1,EstimatedBuildInHours);
		str_cat(is_sTemp3,is_sTemp1);
		str_cat(is_sTemp3,"h ");
	}
	
	//Get Minutes
	if (EstimatedBuildInMinutes>0)
	{
		str_for_num(is_sTemp1,EstimatedBuildInMinutes);
		str_cat(is_sTemp3,is_sTemp1);
		
		if (EstimatedBuildInHours>0) {str_cat(is_sTemp3,"m ");} else {
			str_cat(is_sTemp3,"min ");}
	}
	
	//Get Seconds
	if (EstimatedBuildInSeconds>0)
	{
		str_for_num(is_sTemp1,EstimatedBuildInSeconds);
		str_cat(is_sTemp3,is_sTemp1);
		
		if (EstimatedBuildInHours>0) {str_cat(is_sTemp3,"s ");} else {
			if (EstimatedBuildInMinutes>0) {str_cat(is_sTemp3,"sec ");} else {
				str_cat(is_sTemp3," seconds");}}
	}
}




//Create the Points Of Visibility Graph
function Create_PoVGraph(Total_WayPoints)
{
	var i;
	var CurrentWayPoint=0;
	
	var WayPointDistance;

	var StartingTicks;
	var EdgesAfterRefresh=0;
	
	var EdgesExamined=0;
	var EdgeCandidates;
	var TicksPerEdge;
	var EdgesPerSecond=infinity;

	
	StartingTicks=total_ticks;
	
	
	EdgeCandidates=NumOfWayPoints*NumOfWayPoints;

	
	//Loop until All WayPoints have been examined
	while (CurrentWayPoint<=Total_WayPoints)
	{
		//Test For Links For this Waypoints with all the rest
		i=0;
		while (i<=Total_WayPoints)
		{
			//Make Sure the Waypoint is not trying to find a link with itself
			if (CurrentWayPoint!=i)
			{
				//DO C_TRACE
				is_tempent3=waypoint_to_ptr(CurrentWayPoint);
				is_tempent4=waypoint_to_ptr(i);
				
				//Test For a Walk Link between the two waypoints
				WayPointDistance=TestWalkLinkBetweenTwoPoints(is_tempent3.x,is_tempent4.x,my._max_force_x__003,my);
				EdgesExamined+=1;
				EdgesAfterRefresh+=1;
				
				//If a Path between the two waypoints exists
				if (WayPointDistance!=undefined)
				{
					//Debug-->Draw All Edges Made
					Particle00_DrawPermLine(is_tempent3.x,is_tempent4.x);

					//Print The Edge To the Matrix
					NumOfWayEdges+=1;
					AddGraphEdgeToMatrix(CurrentWayPoint,i,WayPointDistance);
				}
				
				
				//Pick Speciment(For Graph Making Time Data)
				if (EdgesExamined==20)
				{
					wait(1);
					TicksPerEdge=(total_ticks-StartingTicks)/EdgesExamined;
					EdgesPerSecond=is_SecondInTicks/TicksPerEdge;
					
					EstimateTimeRemaining(NumOfWayPoints*NumOfWayPoints*TicksPerEdge);
					str_cpy(sGraphTimeEstimated,is_sTemp3);
				}
				
				
				//Refresh Data Every Second (For Graph Making Time Data)
				if (EdgesAfterRefresh>=EdgesPerSecond)
				{
					EstimateTimeRemaining(((NumOfWayPoints*NumOfWayPoints)-EdgesExamined)*TicksPerEdge);
					str_cpy(sGraphTimeRemaining,is_sTemp3);
					
					vGraphPercentageDone=is_ai_percentage(EdgesExamined,NumOfWayPoints*NumOfWayPoints);
					GraphBar_Percent(vGraphPercentageDone);
					
					EdgesAfterRefresh=0;
					wait(1);
				}
				
				
			}
			i+=1;
		}
		CurrentWayPoint+=1;
	}
	
	bGraphDone=IS_TRUE;
}



//Add the Edge Found to the Points of Visibility Matrix
function AddGraphEdgeToMatrix(prevNode,nextNode,cost)
{
	PoVMatrix[nextNode*MaxWayPoints+prevNode]=cost;
}


//Export the Graph From the Matrix into a .gph File(asciik, may be opened with Notepad for Review)
function ExportGraphToFile(Total_WayPoints)
{
	str_cpy(is_sTemp3,level_name);
	str_trunc(is_sTemp3,3);
	str_cat(is_sTemp3,"gph");
	
	var fHandle;
	
	fHandle=file_open_write(is_sTemp3);
	
	if (fHandle==0) {breakpoint; temp=19;}
	
	
	
	//Export Graph Array
	var i=0;
	var j=0;
	while (i<=Total_WayPoints)
	{
		j=0;
		while (j<=Total_WayPoints)
		{
			file_var_write (fHandle, PoVMatrix[i*MaxWayPoints+j]);
			j+=1;
		}
		i+=1;
	}
	
	file_close (fHandle);
}


//Import a Graph from a .gph file, to the Points of Visibility Matrix
function ImportGraphFromFile(Total_WayPoints)
{
	str_cpy(is_sTemp3,level_name);
	str_trunc(is_sTemp3,3);
	str_cat(is_sTemp3,"gph");
	
	var fHandle;
	
	fHandle=file_open_read(is_sTemp3);
	
	if (fHandle==0) {breakpoint; temp=18;}
	
	
	//Export Graph Array
	var i=0;
	var j=0;
	while (i<=Total_WayPoints)
	{
		j=0;
		while (j<=Total_WayPoints)
		{
			PoVMatrix[i*MaxWayPoints+j]=file_var_read(fHandle);
			j+=1;
		}
		i+=1;
	}
	
	file_close (fHandle);

}




//Important Function. Finds the Waypoint closest to vecPosition. Tests a Walk Link depending on direction(From Waypoint to vecPosition or visa versa)
//It checks all Registered Waypoint Positions and attempts to find a walk link with the closest one. If no link find, it goesto the second closest
//It will give up on finding a Wapoint after 'TotalClosestWaypointsExamined' waypoints have been examined
var WayPointsDistances[MaxWayPoints];
var GetWayPointClosestTo_vecPosition[3];
define dir_PosToWaypoint,1;
define dir_WaypointToPos,2;
function GetWayPointClosestTo(&vecPosition,Total_WayPoints,direction,TotalClosestWayPointsExamined)
{
	//var MinDistance=infinity;
	var MinDistanceWayPointID=undefined;
	
	var WayPointDistance;

	
	GetWayPointClosestTo_vecPosition.x=vecPosition[0];
	GetWayPointClosestTo_vecPosition.y=vecPosition[1];
	GetWayPointClosestTo_vecPosition.z=vecPosition[2];
	
	//For All Waypoints
	var i=0;
	while (i<=Total_WayPoints)
	{
		//Get Distance between the waypoint and VecPosition
		you=waypoint_to_ptr(i);
		WayPointsDistances[i]=vec_dist(you.x,GetWayPointClosestTo_vecPosition.x);
		i+=1;	
	}
	
	
	//OPTIMIZATON: Halt this function if no eligible Waypoint found after the first closest X ones(X==TotalClosestWayPointsExamined)
	i=0;
	while (i<TotalClosestWayPointsExamined) 
	{
		//Extract the Waypoint with the shortes distance(closest to vecPosition)
		MinDistanceWayPointID=ExtractMinimumFrom(WayPointsDistances,Total_WayPoints,infinity);
		
		//Get its pointer
		you=waypoint_to_ptr(MinDistanceWayPointID);
		
		
		//Test for a Walk Link From vecPosition--->Waypoint
		if (direction==dir_PosToWaypoint)
		{
			WayPointDistance=TestWalkLinkBetweenTwoPoints(GetWayPointClosestTo_vecPosition.x,you.x,vec_dist(GetWayPointClosestTo_vecPosition.x,you.x)/is_ai_Pathfind_OptimizedSimulatedSteps,is_cIgnoreAllDynamicModels);
		}
		//Test for a Walk Link From WayPoint--->vecPosition
		else 
		{
			WayPointDistance=TestWalkLinkBetweenTwoPoints(you.x,GetWayPointClosestTo_vecPosition.x,vec_dist(you.x,GetWayPointClosestTo_vecPosition.x)/is_ai_Pathfind_OptimizedSimulatedSteps,is_cIgnoreAllDynamicModels);
		}

		//If Walk Link Found
		if (WayPointDistance!=undefined)
		{
			//Return Waypoint and it's distance
			GetWayPointClosestTo_ReturnDistance=WayPointDistance;
			return(MinDistanceWayPointID);
		}
		
		i+=1;
	}
	
	
	//If it comes here, No WayPoint is visible from Start or Target Point!!! Return Undefined Waypoint ID!!
	return(undefined);
}



//Shorten the Pathfinded Path by removing unesseccery waypoints from it
//Dijkstra always returns the optimal Path found, so no String Pulling is nesseccery in the actual Path
//However, Npc's Position May be able to be linked directly with the 2nd Waypoint from the path and not with the closest WayPoint to it(marked as yellow)
//On the same note, the Target Goal Position may be able to be linked with the second to last waypoyint from the path and not with the closest Waypoint to it(marked as blue)
//On Super Debug mode, if you notice the path doesn't connect the Yellow or Blue Waypoints, it means String Pulling has optimized the Path
function StringPulling(&PathArray,eIgnored)
{
	var WayPointDistance;

	
	//Find Last Node Index Number
	var i=0;
	while (PathArray[my.is_vIndex*MaxWayPoints+i]!=undefined)
	{
		i+=1;
	}
	i-=1;
	
	
	//Pulling Out First WayPoint

	//Test for Walk Link between Npc and 2nd WayPoint
	you=waypoint_to_ptr(PathArray[my.is_vIndex*MaxWayPoints+1]);
	WayPointDistance=TestWalkLinkBetweenTwoPoints(my.x,you.x,vec_dist(my.x,you.x)/is_ai_Pathfind_OptimizedSimulatedSteps,is_cIgnoreAllDynamicModels);

	//If Walk Link Found, modify the path to exclude 1st WayPoint
	if (WayPointDistance!=undefined) {PathArray[my.is_vIndex*MaxWayPoints+0]=PathArray[my.is_vIndex*MaxWayPoints+1];}


	//Pulling Out Last WayPoint
	
	
	//Test for Walk Link between 2nd to last Waypoint and Goal Target Position
	you=waypoint_to_ptr(PathArray[my.is_vIndex*MaxWayPoints+i-1]);
	WayPointDistance=TestWalkLinkBetweenTwoPoints(you.x,my.goal_position_x,vec_dist(you.x,my.goal_position_x)/is_ai_Pathfind_OptimizedSimulatedSteps,is_cIgnoreAllDynamicModels);

	//If Walk Link Found, modify the path to exclude last Waypoint
	if (WayPointDistance!=undefined) {PathArray[my.is_vIndex*MaxWayPoints+i]=PathArray[my.is_vIndex*MaxWayPoints+i-1];}
}



/*********************************************
**********************************************
**            DEBUGGING FUNCTIONS           **
**********************************************
*********************************************/


//WayPoint Colors
define WayPointSkin_Green,1;
define WayPointSkin_Red,2;
define WayPointSkin_Blue,3;
define WayPointSkin_Yellow,4;


//Mark WayPoint Green
function MarkGreenWayPointId(WayPointId)
{
	you=waypoint_to_ptr(WayPointId);
	
	you.skin=WayPointSkin_Green;
}

//Mark WayPoint Red
function MarkRedWayPointId(WayPointId)
{
	you=waypoint_to_ptr(WayPointId);
	
	you.skin=WayPointSkin_Red;
}

//Mark WayPoint Blue
function MarkBlueWayPointId(WayPointId)
{
	you=waypoint_to_ptr(WayPointId);
	
	you.skin=WayPointSkin_Blue;
}

//Mark WayPoint Yellow
function MarkYellowWayPointId(WayPointId)
{
	you=waypoint_to_ptr(WayPointId);
	
	you.skin=WayPointSkin_Yellow;
}

//Toggle Visibility of all Registered Waypoints
function ToggleWaypoints(Visible)
{
	var i;
	
		//Debug-->Make WayPoints Visible or Not
		i=0;
		while (i<=waypoint_count)
		{
			you=waypoint_to_ptr(i);
			if (Visible) {you.invisible=off;} else {you.invisible=on;}
			i+=1;
		}
}


//Debug Panel(currently shows the number of total waypoints if set to visible)
panel misc_debug
{
	layer=50;
	pos_x = 300;
	pos_y = 5;
	flags = refresh;//,visible;

	digits 0,0,5,Arial_font,1,NumOfWayPoints;
}



//Panel Displaying a small portion of the PoVMatrix. Used to visually understand how Dijkstra works, by using no more than 6 total WayPoints(to fit into the debugging Panel)
panel PoVmatrix_debug
{
	layer=50;
	pos_x = 600;
	pos_y = 20;
	flags = refresh;//,visible;

	//Line x0
	digits 100,100,5,Arial_font,1,PoVMatrix[0];
	digits 100,120,5,Arial_font,1,PoVMatrix[1];
	digits 100,140,5,Arial_font,1,PoVMatrix[2];
	digits 100,160,5,Arial_font,1,PoVMatrix[3];
	digits 100,180,5,Arial_font,1,PoVMatrix[4];
	digits 100,200,5,Arial_font,1,PoVMatrix[5];

	//Line x1
	digits 120,100,5,Arial_font,1,PoVMatrix[1000];
	digits 120,120,5,Arial_font,1,PoVMatrix[1001];
	digits 120,140,5,Arial_font,1,PoVMatrix[1002];
	digits 120,160,5,Arial_font,1,PoVMatrix[1003];
	digits 120,180,5,Arial_font,1,PoVMatrix[1004];
	digits 120,200,5,Arial_font,1,PoVMatrix[1005];

	//Line x2
	digits 140,100,5,Arial_font,1,PoVMatrix[2000];
	digits 140,120,5,Arial_font,1,PoVMatrix[2001];
	digits 140,140,5,Arial_font,1,PoVMatrix[2002];
	digits 140,160,5,Arial_font,1,PoVMatrix[2003];
	digits 140,180,5,Arial_font,1,PoVMatrix[2004];
	digits 140,200,5,Arial_font,1,PoVMatrix[2005];

	//Line x3
	digits 160,100,5,Arial_font,1,PoVMatrix[3000];
	digits 160,120,5,Arial_font,1,PoVMatrix[3001];
	digits 160,140,5,Arial_font,1,PoVMatrix[3002];
	digits 160,160,5,Arial_font,1,PoVMatrix[3003];
	digits 160,180,5,Arial_font,1,PoVMatrix[3004];
	digits 160,200,5,Arial_font,1,PoVMatrix[3005];

	//Line x4
	digits 180,100,5,Arial_font,1,PoVMatrix[4000];
	digits 180,120,5,Arial_font,1,PoVMatrix[4001];
	digits 180,140,5,Arial_font,1,PoVMatrix[4002];
	digits 180,160,5,Arial_font,1,PoVMatrix[4003];
	digits 180,180,5,Arial_font,1,PoVMatrix[4004];
	digits 180,200,5,Arial_font,1,PoVMatrix[4005];

	//Line x5
	digits 200,100,5,Arial_font,1,PoVMatrix[5000];
	digits 200,120,5,Arial_font,1,PoVMatrix[5001];
	digits 200,140,5,Arial_font,1,PoVMatrix[5002];
	digits 200,160,5,Arial_font,1,PoVMatrix[5003];
	digits 200,180,5,Arial_font,1,PoVMatrix[5004];
	digits 200,200,5,Arial_font,1,PoVMatrix[5005];

}