/*********************************************************************
**									Intense Pathfinding                      **
**                         -------------------                      **
**                                             By Intense Illusions **
*********************************************************************/




/*********************************************
**********************************************
**                 NPC ACTONS               **
**********************************************
*********************************************/

//These actions take more than a wait cycle to be executed. Npc use them to achive certain goals


//Npc Moves on a Path which was generated by a Pathfinding routine
function is_Action_Move_On_Path(&pathArray,IsDirect)
{
	var WayPointFollowing=0;

	//If Direct Movement With No Pathfinding Involved, move directly to Goal
	if (IsDirect)
	{
		WayPointFollowing=is_ai_GoalWayPoint;
	}
	
	//Indicate the Npc is now moving
	my.is_cMoveMode=is_cMoving;
	
	//While the Npc is moving
	while (my.is_cMoveMode==is_cMoving)
	{
		//Set the Id Of the WayPoint To Move To Next
		WayPointFollowing=is_SubAction_SetNextWayPoint(pathArray,WayPointFollowing,is_ai_WayPointTriggerRange*is_meter);
		
		//Check If Path Has Ended
		if (WayPointFollowing==is_ai_PathEnded) {break;}
		
		//Move Towards Target WayPoint for 1 frame
		is_SubAction_MoveTowardsWayPoint(pathArray,WayPointFollowing);
		
		wait(1);
	}
	
	//If it reached this point, it means the Npc reached its destination, so stop it.
	my.is_cMoveMode=is_cStopped;
}




//Main Function which moves an Npc('my' pointer) to PosX,Y,Z  OR  to eTarget
//Use this function to move an npc somewhere
//Returns TRUE(1) if a path was successfully found, or FALSe(0) otherwise
function is_Action_Move_To(PosX,PosY,PosZ,eTarget)
{
	var StartWayPoint;
	var TargetWayPoint;
	var PathDistance;
	var i;
	var WayPointDistance;
	var MyOriginHeight;
	
	
	is_ai_Pathfind_OptimizedSimulatedSteps=is_ai_cGravitySteps;
	
	
	//Store Goal Coordinates On Entity
	if (eTarget==null)
	{
		you=null;
		my.goal_position_x=PosX;
		my.goal_position_y=PosY;
		my.goal_position_z=PosZ;
	}
	else
	{
		you=eTarget;
		
		MyOriginHeight=my.min_z*(-1);
		
		ClampPosition(you.x,MyOriginHeight,is_temp3);
		
		my.goal_position_x=is_temp3.x;
		my.goal_position_y=is_temp3.y;
		my.goal_position_z=is_temp3.z;
	}
	
	
	
	//Debug-->Reset All WayPoints To Green
	if (is_ai_bDebugMode)
	{
		i=0;
		while (i<=waypoint_count)
		{
			MarkGreenWayPointId(i);
			i+=1;
		}
	}
	
	
	//Do TestWalkLink to check for direct Path between Agent and Target
	WayPointDistance=TestWalkLinkBetweenTwoPoints(my.x,my.goal_position_x,vec_dist(my.x,my.goal_position_x)/is_ai_Pathfind_OptimizedSimulatedSteps,is_cIgnoreAllDynamicModels);

	
	//If Path Between Agent and Target, Move Directly
	if (WayPointDistance!=undefined)
	{
			//Debug-->Draw Path
		//Draw Edge From Agent To Target
		if (is_ai_bDebugMode) {Particle00_DrawPermLine(my.x,my.goal_position_x);}
		
		//Walk the Path
		is_Action_Move_On_Path(PoV_PathsDataBase,IS_TRUE);
		
		return(IS_TRUE);
	}
	

	//Find WayPoint For StartPoint
	StartWayPoint=GetWayPointClosestTo(my.x,waypoint_count,dir_PosToWaypoint,10);
	
	//Find WayPoint For TargetPoint
	TargetWayPoint=GetWayPointClosestTo(my.goal_position_x,waypoint_count,dir_WaypointToPos,10);


	//Halt Movement if Agent's OR Target Point is UnReachable
	if (StartWayPoint==undefined || TargetWayPoint==undefined)
	{
		my.is_cMoveMode=is_cStopped;

		return(IS_FALSE);
	}
	

		//Debug-->Highlight StartWayPoint and TargetWayPoint
		if (is_ai_bDebugMode)
		{
			MarkYellowWayPointId(StartWayPoint);
			MarkBlueWayPointId(TargetWayPoint);
		}

		

	//Do Dijsktra
	Dijkstra(PoVMatrix,TargetWayPoint,waypoint_count,MaxWayPoints);
	
	
	//GetPath
	PathDistance=GetShortestPathByIteration2dArray(Dijkstra_previous,PoV_PathsDataBase,StartWayPoint,MaxWayPoints);
	

	
	//Halt Movement if No Path is Found
	if (PathDistance==undefined)
	{
		my.is_cMoveMode=is_cStopped;

		return(IS_FALSE);
	}
	
	
	
	//Debug-->Highlight WayPoints Included in Path
	if (is_ai_bDebugMode)
	{
		i=0;
		while (PoV_PathsDataBase[my.is_vIndex*MaxWayPoints+i]!=undefined)
		{
			is_tempent=waypoint_to_ptr(PoV_PathsDataBase[my.is_vIndex*MaxWayPoints+i]);
			if (is_tempent.skin==WayPointSkin_Green)
			{
				MarkRedWayPointId(PoV_PathsDataBase[my.is_vIndex*MaxWayPoints+i]);
			}
			i+=1;
		}
	}
	
	
	//StringPulling
	if (StartWayPoint!=TargetWayPoint) //Only attempt String Pulling if there's more one than Waypoints involved)
	{
		StringPulling(PoV_PathsDataBase,eTarget);
	}
	
	
	
	//Debug-->Draw Path
	if (is_ai_bDebugMode)
	{
		//Draw Edge From Agent To First WayPoint
		is_tempent=waypoint_to_ptr(PoV_PathsDataBase[my.is_vIndex*MaxWayPoints+0]);
		Particle00_DrawPermLine(vector(my.x,my.y,my.z),is_tempent.x);
		
		//Draw Edges Between WayPoints
		i=1;
		while (PoV_PathsDataBase[my.is_vIndex*MaxWayPoints+i]!=undefined)
		{
			is_tempent=waypoint_to_ptr(PoV_PathsDataBase[my.is_vIndex*MaxWayPoints+i-1]);
			is_tempent2=waypoint_to_ptr(PoV_PathsDataBase[my.is_vIndex*MaxWayPoints+i]);
			Particle00_DrawPermLine(is_tempent.x,is_tempent2.x);

			i+=1;
		}
		
		//Draw Edge From Last WayPoint To Goal Pos
		is_tempent=waypoint_to_ptr(PoV_PathsDataBase[my.is_vIndex*MaxWayPoints+i-1]);
		Particle00_DrawPermLine(is_tempent.x,my.goal_position_x);
	}


	
	//Walk the Path
	is_Action_Move_On_Path(PoV_PathsDataBase,IS_FALSE);
	

	return(IS_TRUE);
}

