/*********************************************************************
**									Intense Pathfinding                      **
**                         -------------------                      **
**                                             By Intense Illusions **
*********************************************************************/




/*********************************************
**********************************************
**              NPC SUBACTONS               **
**********************************************
*********************************************/


//This is actions that the Npc uses to accomplish a (long term) action. They will not take more than a cycle to be executed (no waits)



//Returns Pathfinded Distance between My Entity and X or undefined if path not found
	//Similar to is_Action_Move_To(PosX,PosY,PosZ,eTarget)
//This is an optimized function which will generate a path between two points
	//but won't move the Npc. It will return the pathfinded distance between 'my' and PosX,Y,Z OR eTarget
//This function is not used in this Pathfinding demo, but you may find it usefull later on(keep it in mind:) )
function is_SubAction_getDistance_To(PosX,PosY,PosZ,eTarget)
{
	var StartWayPoint;
	var TargetWayPoint;
	var PathDistance;
	var i;
	var WayPointDistance;
	var TotalDistance=0;
	
	is_ai_Pathfind_OptimizedSimulatedSteps=1;


	//Store Goal Coordinates On Entity
	if (eTarget==null)
	{
		you=null;
	}
	else
	{
		you=eTarget;
		PosX=you.x;
		PosY=you.y;
		PosZ=you.z;
	}
	
	
	
	//Do TestWalkLink to check for direct Path between Agent and Target
	WayPointDistance=TestWalkLinkBetweenTwoPoints(my.x,vector(PosX,PosY,PosZ),vec_dist(my.x,vector(PosX,PosY,PosZ))/is_ai_Pathfind_OptimizedSimulatedSteps,is_cIgnoreAllDynamicModels);
	
	
	//If Path Between Agent and Target, Move Directly
	if (WayPointDistance!=undefined)
	{
		//Return Distance
		return(WayPointDistance);
	}
	

	//Find WayPoint For StartPoint
	StartWayPoint=GetWayPointClosestTo(my.x,waypoint_count,dir_PosToWaypoint,2);
	
	//Note Distance From StartPoint To First WayPoint
	TotalDistance+=GetWayPointClosestTo_ReturnDistance;
	
	
	
	//Find WayPoint For TargetPoint
	TargetWayPoint=GetWayPointClosestTo(vector(PosX,PosY,PosZ),waypoint_count,dir_WaypointToPos,2);
	
	//Note Distance From Last WayPoint To TargetPoint
	TotalDistance+=GetWayPointClosestTo_ReturnDistance;



	//Halt Movement if Agent's OR Target Point is UnReachable
	if (StartWayPoint==undefined || TargetWayPoint==undefined)
	{
		return(undefined);
	}


	//Do Dijsktra
	Dijkstra(PoVMatrix,TargetWayPoint,waypoint_count,MaxWayPoints);
	
	
	//GetPath Distance
	PathDistance=Dijkstra_costs[StartWayPoint];
	
	
	//Halt Movement if No Path is Found
	if (PathDistance==infinity)
	{
		return(undefined);
	}
	else
	{
		//Note Distance For Path Between First and Last WayPoints
		TotalDistance+=PathDistance;
	}

	
	//Return Distance
	return(TotalDistance);
}





// Move eEntity To face panTo
function	is_SubAction_PanTo(eEntity,panTo)
{
	my=eEntity;

	// turn to face panTo
	if(panTo > my._max_ang_force_pan__003*TIME)
	{
		// turn anti-clockwise
		my.PanForce=my._max_ang_force_pan__003*TIME;
	}
	else
	{
		if(panTo < -my._max_ang_force_pan__003*TIME)
		{
			// turn anti-clockwise
			my.PanForce=-my._max_ang_force_pan__003*TIME;
		}
		else
		{
			// turn "exactly" to
			my.PanForce=panTo;
		}
	}

	return(my.pan+my.PanForce);
}




//Sets the next Waypoint on to the Path generated by the pathfinding routine.
function is_SubAction_SetNextWayPoint(pathArray,WayPointFollowing,TriggerRadious)
{
	var WayPointId;
	
	
	//If WayPoint Is Not the Goal Node
	if (WayPointFollowing!=is_ai_GoalWayPoint)
	{
		WayPointId=pathArray[my.is_vIndex*MaxWayPoints+WayPointFollowing];
		
		you=waypoint_to_ptr(WayPointId);
		
		if (vec_dist(my.x,you.x)<=TriggerRadious)
		{
			WayPointFollowing+=1;
		}
		
		//Check if Next WayPoint is Goal Node
		if (pathArray[my.is_vIndex*MaxWayPoints+WayPointFollowing]==undefined) {WayPointFollowing=is_ai_GoalWayPoint;}
	}
	//If Waypoint is the Goal Node
	else
	{
		if (vec_dist(vector(my.x,my.y,0),vector(my.goal_position_x,my.goal_position_y,0))<=TriggerRadious)  //Ignore Goal's Height. Assume Goal reached, even if actual goal is behind a staircase or something. Dijkstra should never let this happen.
		{
			WayPointFollowing=is_ai_PathEnded;
		}
	}
	
	//Return the Waypoint Found
	return(WayPointFollowing);
}



//Make a 1-cycle move towards a Waypoint
function is_SubAction_MoveTowardsWayPoint(pathArray,WayPointFollowing)
{
	var WayPointId;
	var AngleToWayPoint;
	var DistanceToWayPoint;
	var AngleOffset;
	var MyNewPan[3];
	
	
	//If WayPoint Is Not the Goal Node
	if (WayPointFollowing!=is_ai_GoalWayPoint)
	{
		WayPointId=pathArray[my.is_vIndex*MaxWayPoints+WayPointFollowing];
		
		you=waypoint_to_ptr(WayPointId);
	
		AngleToWayPoint=is_ReturnAngleForTurningToEntity(my,you);
		DistanceToWayPoint=vec_dist(my.x,you.x);
	}
	//If Waypoint is the Goal Node
	else
	{
		AngleToWayPoint=is_ReturnAngleForTurningToPos(my,my.goal_position_x,my.goal_position_y,my.goal_position_z);
		DistanceToWayPoint=vec_dist(my.x,vector(my.goal_position_x,my.goal_position_y,my.goal_position_z));
	}


	//TIP: AngleToWaypoint now contains the direction the Bot should move to in order to honour its path
	
	
	//Enforce A Obstacle Avoidance Method
	
	//Obstacle Avoidance Using c_traces and Recursion (inactive)
	//AngleToWayPoint=is_SubAction_ObstacleAvoidanceCtrace(AngleToWayPoint,DistanceToWayPoint,0);
	
	//Obstacle Avoidance Using Repulsive Vectors (disabled)
	//AngleToWayPoint=is_SubAction_ObstacleAvoidanceRepulsionVecs(AngleToWayPoint);

	//If Bot's Direction to Waypoint is 
	if (AngleToWayPoint!=undefinedAngle)
	{
			//Handle Steering Force
			AngleOffset = ang(AngleToWayPoint - my.pan);
			is_SubAction_PanTo(my,AngleOffset);
	
			//Handle Movement Force
			vec_set(my.boostVectorX,vector(my._max_force_x__003,0,0)); // move forward
	}
	else
	{
		//Handle this in another layer (when this flag is raised, try to get the Npc unstuck and then attempt to pathfind again)
		my.is_bIsStuck=IS_TRUE;
	}
	
}

